# PA165 Movie Recommender Catalogue

*PA165 course at MUNI Spring 2023.*

# Project description

The web application is a catalogue of movies of different genres. 
Each movie has a title, description, film director, and list of actors, and image(s) from the main playbill. 
Each movie is categorized in one or more genres. 
Only the administrator can create, remove and update the list of movies. 

Users can register and log in to get access to the application. 
The administrator account is created by the system.

Users can rate the movies according to different criteria 
(e.g how novel are the ideas of the movie, their final score, etc…). 

The main feature of the system is that users can pick one movie 
and get the list of similar movies and / or movies that were liked the most by other users watching the same movie.

Also, users can generate reports consisting of all movies ordered by average rating.

## Services description

Each service is designated to do operations that are common i some way.

### Client service
This client service is used for authorization.

### Core service
This service stores all the data. Other services calls it to get the data.

Runs on localhost:8080

### Recommender service
This service generates a list of movies that are similar with the given movie.

Runs on localhost:8081

### Report service
This service generates files of movies ordered by average rating

Runs on localhost:8082

# How to run Movie Recommender Catalogue
There are two ways how to run the application. With docker or with maven.

## Maven
In order to run Movie Recommender Catalogue with maven, first run in the main project folder:

    mvn clean install

Then run these commands to switch to each of the executable module and run the application. 
Run each set of commands in its own terminal window.

Run client application:

    cd client
    mvn spring-boot:run

Run core application:

    cd core
    mvn spring-boot:run

Run core recommender application:

    cd recommender
    mvn spring-boot:run

Run core report application:

    cd report
    mvn spring-boot:run

## How to login and use app

- run all application in separate terminals
- go to localhost:8080

![img_1.png](src/diagrams/images/img_1.png)

- click on Login MUNI
- let everything checked (or check what you need for testing)

![img_3.png](src/diagrams/images/img_3.png)

- after login click on GET TOKEN

![img_4.png](src/diagrams/images/img_4.png)

- copy token for swagger

![img_5.png](src/diagrams/images/img_5.png)

- in swagger click on Authorize and paste token


## Docker
In order to run Movie Recommender Catalogue in docker, first run in the main project folder:

    mvn clean install

Then start the application with docker:

    docker-compose up

## Scenario
There is a python script, that simulates application usage.
In the scenario, first all ratings, users and movies are deleted.
Then a few new users and movies are created.
Then, to test server load, a lot of random ratings are created in parallel.
And as a demonstration of the recommender service, a few recommendations of similar movies are requested.

To run the scenario script, first start the application with maven or docker and then run the script with python interpreter:

    python .\src\scenario-scripts\scenario.py

And the results are printed in the console, together with info about created entities.
# Diagrams
![use-case-diagram.png](src/diagrams/images/use-case-diagram.png)
![class-diagram.png](src/diagrams/images/class-diagram.png)

# Interaction with backend:
DB: http://localhost:8080/h2-console

and log in with credentials from
src/main/resources/application.yml

Swagger: http://localhost:8080/swagger-ui/index.html

