package cz.muni.fi.pa165.movie;

import cz.muni.fi.pa165.common.BaseService;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class MovieService extends BaseService<MovieDo> {

    @Getter
    private final MovieRepository repository;

    @Getter
    private final MovieMapper movieMapper;

    @Autowired
    public MovieService(MovieRepository movieRepository, MovieMapper movieMapper) {
        this.repository = movieRepository;
        this.movieMapper = movieMapper;
    }


    @Transactional(readOnly = true)
    public List<MovieDto> getMovies () {
        return repository.findAll().stream().map(movieMapper::toDto).toList();
    }

    @Transactional(readOnly = true)
    public MovieDto getMovie(long id) {
        MovieDo movie = repository.getReferenceById(id);
        return movieMapper.toDto(movie);
    }

    @Transactional
    public void createMovie(MovieDto movieDto) {
        MovieDo movie = movieMapper.fromDto(movieDto);
        repository.save(movie);
    }

    @Transactional
    public void updateMovie(long id, MovieDto movieDto) {
        movieMapper.updateFromDto(movieDto, repository.getReferenceById(id));
    }

    @Transactional
    public void deleteMovie(long id) {
        repository.delete(repository.getReferenceById(id));
    }

    @Transactional
    public void deleteAllMovies() {
        repository.deleteAll();
    }

    @Transactional(readOnly = true)
    public MovieDto findByTitle(String title) {
        return movieMapper.toDto(repository.findByTitle(title));
    }
}
