package cz.muni.fi.pa165.rating;

import cz.muni.fi.pa165.CoreApplication;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(path = "/ratings", produces = MediaType.APPLICATION_JSON_VALUE)
public class RatingController {

    private final RatingService ratingService;

    @Autowired
    public RatingController(RatingService ratingService) {
        this.ratingService = ratingService;
    }

    @Operation(summary = "Get all ratings",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_read"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - ratings were successfully returned"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_read", content = @Content()),
            }
    )
    @GetMapping
    public List<RatingDto> getRatings() {
        return ratingService.getRatings();
    }

    @Operation(summary = "Create new rating",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_write"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - rating was successfully created"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_write", content = @Content()),
            }
    )
    @PostMapping
    public void createRating(@Valid @RequestBody RatingDto ratingDto) {
        ratingService.createRating(ratingDto);
    }

    @Operation(summary = "Update rating",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_write"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - rating was successfully updated"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_write", content = @Content()),
            }
    )
    @PostMapping("{id}")
    public void updateRating(@PathVariable long id, @Valid @RequestBody RatingDto ratingDto) {
        ratingService.updateRating(id, ratingDto);
    }

    @Operation(summary = "Delete rating with id",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_1"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - rating was successfully deleted"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_1", content = @Content()),
            }
    )
    @DeleteMapping("{id}")
    public void deleteRating(@PathVariable long id) {
        ratingService.deleteRating(id);
    }

    @Operation(summary = "Delete all ratings",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_1"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - ratings were successfully deleted"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_1", content = @Content()),
            }
    )
    @DeleteMapping
    public void deleteAllRatings() {
        ratingService.deleteAllRatings();
    }

    @Operation(summary = "Get rating by id",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_read"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - rating was successfully returned"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_read", content = @Content()),
            }
    )
    @GetMapping("{id}")
    public RatingDto getById(@PathVariable long id) {
        return ratingService.getById(id);
    }
}
