package cz.muni.fi.pa165.rating;

import cz.muni.fi.pa165.common.BaseService;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class RatingService extends BaseService<RatingDo> {

    @Getter
    private final RatingRepository repository;

    @Getter
    private final RatingMapper ratingMapper;

    @Autowired
    public RatingService(RatingRepository ratingRepository, RatingMapper ratingMapper) {
        this.repository = ratingRepository;
        this.ratingMapper = ratingMapper;
    }

    @Transactional(readOnly = true)
    public List<RatingDto> getRatings() {
        return repository.findAll().stream().map(ratingMapper::toDto).toList();
    }

    @Transactional
    public void createRating(RatingDto ratingDto) {
        RatingDo rating = ratingMapper.fromDto(ratingDto);
        repository.save(rating);
    }

    @Transactional
    public void updateRating(long id, RatingDto ratingDto) {
        ratingMapper.updateFromDto(ratingDto, repository.getReferenceById(id));
    }

    @Transactional
    public void deleteRating(long id) {
        repository.delete(repository.getReferenceById(id));
    }

    @Transactional(readOnly = true)
    public RatingDto getById(long id) {
        return ratingMapper.toDto(repository.getReferenceById(id));
    }

    @Transactional
    public void deleteAllRatings() {
        repository.deleteAll();
    }
}
