package cz.muni.fi.pa165.user;

import cz.muni.fi.pa165.CoreApplication;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@RestController
@RequestMapping(path = "/users", produces = MediaType.APPLICATION_JSON_VALUE)
public class UserController {

    private final UserService userService;


    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @Operation(summary = "Create new user",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_write"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - user was successfully created"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_write", content = @Content()),
            }
    )
    @PostMapping
    public void createUser(@Valid @RequestBody UserDto userDto) {
        userService.createUser(userDto);
    }

    @Operation(summary = "Update user",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_write"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - user was successfully updated"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_write", content = @Content()),
            }
    )
    @PostMapping("{id}")
    public void updateUser(@PathVariable long id, @Valid @RequestBody UserDto userDto) {
        userService.updateUser(id, userDto);
    }

    @Operation(summary = "Delete user with id",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_1"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - user was successfully deleted"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_1", content = @Content()),
            }
    )
    @DeleteMapping("{id}")
    public void deleteUser(@PathVariable long id) {
        userService.deleteUser(id);
    }

    @Operation(summary = "Delete all users",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_1"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - users were successfully deleted"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_1", content = @Content()),
            }
    )
    @DeleteMapping
    public void deleteAllUsers() {
        userService.deleteAllUsers();
    }

    @Operation(summary = "Get user by username",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_read"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - user was successfully returned"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_read", content = @Content()),
            }
    )
    @GetMapping("{username}")
    public UserDto findByUsername(@PathVariable String username) {
        return userService.findByUsername(username);
    }
}
