package cz.muni.fi.pa165.user;

import cz.muni.fi.pa165.common.BaseService;
import cz.muni.fi.pa165.movie.MovieDto;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class UserService extends BaseService<UserDo> {

    @Getter
    private final UserRepository repository;

    @Getter
    private final UserMapper userMapper;

    @Autowired
    public UserService(UserRepository repository, UserMapper userMapper) {
        this.repository = repository;
        this.userMapper = userMapper;
    }

    @Transactional
    public void createUser(UserDto userDto) {
        UserDo user = userMapper.fromDto(userDto);
        repository.save(user);
    }

    @Transactional
    public void updateUser(long id, UserDto userDto) {
        userMapper.updateFromDto(userDto, repository.getReferenceById(id));
    }

    @Transactional
    public void deleteUser(long id) {
        repository.delete(repository.getReferenceById(id));
    }

    @Transactional
    public void deleteAllUsers() {
        repository.deleteAll();
    }

    @Transactional(readOnly = true)
    public UserDto findByUsername(String username) {
        return userMapper.toDto(repository.findByUsername(username));
    }
}
