package cz.muni.fi.pa165;

import cz.muni.fi.pa165.image.ImageDo;
import cz.muni.fi.pa165.image.ImageDto;
import cz.muni.fi.pa165.image.ImageMapperImpl;
import cz.muni.fi.pa165.movie.*;
import lombok.Getter;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

@SpringBootTest(classes = CoreApplication.class)
@AutoConfigureMockMvc
@ExtendWith(SpringExtension.class)
public class MovieServiceTest {

    @Getter
    private final MovieMapper movieMapper;

    @Mock
    private MovieRepository movieRepository;

    @InjectMocks
    private MovieService movieService;

    @Autowired
    public MovieServiceTest(MovieMapper movieMapper) {
        this.movieMapper = movieMapper;
    }

    @BeforeEach
    public void setUp() {
        movieService = new MovieService(movieRepository, movieMapper);
    }

    @Test
    public void testGetAllMovies() {
        MovieDo movie1 = MovieDo.builder()
                .title("Example Movie")
                .description("Action")
                .director("John Doe").
                genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.THRILLER))
                .actors(Arrays.asList("Actor1", "Actor2"))
                .build();
        movie1.setId(0L);

        var x = ImageDo.builder().movieId(movie1.getId()).movie(movie1).caption("Naked banana").build();
        x.setId(0L);
        var z = ImageDo.builder().movieId(movie1.getId()).movie(movie1).caption("Naked apple").build();
        z.setId(1L);
        movie1.setImages(Arrays.asList(x, z));

        MovieDo movie2 = MovieDo.builder()
                .title("Example Movie 2")
                .description("Comedy") // Changed description
                .director("Jane Smith") // Changed director
                .genres(Arrays.asList(GenreEnum.COMEDY, GenreEnum.ROMANCE)) // Changed genres
                .actors(Arrays.asList("Actor3", "Actor4")) // Changed actors
                .build();
        movie2.setId(1L);

        var xx = ImageDo.builder().movieId(movie2.getId()).movie(movie2).caption("Naked banana 2").build();
        xx.setId(2L);
        var zz = ImageDo.builder().movieId(movie2.getId()).movie(movie2).caption("Naked apple 2").build();
        zz.setId(3L);
        movie2.setImages(Arrays.asList(xx, zz));


        when(movieRepository.findAll()).thenReturn(List.of(movie1, movie2));

        List<MovieDto> result = movieService.getMovies();

        verify(movieRepository, times(1)).findAll();

        assertThat(result).hasSize(2);

        Optional<MovieDto> movie1ResultOptional = result.stream().filter(dto -> Objects.equals(dto.getTitle(), "Example Movie")).findFirst();
        assertThat(movie1ResultOptional.isEmpty()).isFalse();
        Optional<MovieDto> movie2ResultOptional = result.stream().filter(dto -> Objects.equals(dto.getTitle(), "Example Movie 2")).findFirst();
        assertThat(movie2ResultOptional.isEmpty()).isFalse();

        MovieDto movie1Result = movie1ResultOptional.get();
        assertThat(movie1Result.getId()).isEqualTo(movie1.getId());
        assertThat(movie1Result.getTitle()).isEqualTo(movie1.getTitle());
        assertThat(movie1Result.getDescription()).isEqualTo(movie1.getDescription());
        assertThat(movie1Result.getDirector()).isEqualTo(movie1.getDirector());
        assertThat(movie1Result.getGenres()).isEqualTo(movie1.getGenres());
        assertThat(movie1Result.getActors()).isEqualTo(movie1.getActors());

        var imageMapper = new ImageMapperImpl();
        var imageDtos = movie1.getImages().stream().map(imageMapper::toDto).toList();

        assertThat(imageDtos).isEqualTo(movie1Result.getImages());

//-----------------------------------------------------

        MovieDto movie2Result = movie2ResultOptional.get();
        assertThat(movie2Result.getId()).isEqualTo(movie2.getId());
        assertThat(movie2Result.getTitle()).isEqualTo(movie2.getTitle());
        assertThat(movie2Result.getDescription()).isEqualTo(movie2.getDescription());
        assertThat(movie2Result.getDirector()).isEqualTo(movie2.getDirector());
        assertThat(movie2Result.getGenres()).isEqualTo(movie2.getGenres());
        assertThat(movie2Result.getActors()).isEqualTo(movie2.getActors());

        imageDtos = movie2.getImages().stream().map(imageMapper::toDto).toList();

        assertThat(imageDtos).isEqualTo(movie2Result.getImages());
    }

    @Test
    public void testCreateMovie() {
        MovieDto movieDto = new MovieDto();
        movieDto.setTitle("Example Movie");
        movieDto.setDescription("Action");
        movieDto.setDirector("John Doe");
        movieDto.setGenres(Arrays.asList(GenreEnum.ACTION, GenreEnum.THRILLER));
        movieDto.setActors(Arrays.asList("Actor1", "Actor2"));
        movieDto.setImages(Arrays.asList(
                new ImageDto(),
                new ImageDto()));

        movieService.createMovie(movieDto);

        verify(movieRepository, Mockito.times(1)).save(Mockito.any(MovieDo.class));
    }

    @Test
    public void testGetMovieById() {
        MovieDo movie = MovieDo.builder()
                .title("Example Movie")
                .description("Action")
                .director("John Doe").
                genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.THRILLER))
                .actors(Arrays.asList("Actor1", "Actor2"))
                .build();

        var x = ImageDo.builder().movieId(movie.getId()).movie(movie).caption("Naked banana").build();
        x.setId(0L);
        var z = ImageDo.builder().movieId(movie.getId()).movie(movie).caption("Naked apple").build();
        z.setId(1L);
        movie.setImages(Arrays.asList(x, z));

        when(movieRepository.getReferenceById(movie.getId())).thenReturn(movie);

        MovieDto result = movieService.getMovie(movie.getId());

        verify(movieRepository, times(1)).getReferenceById(movie.getId());

        assertThat(result.getId()).isEqualTo(movie.getId());
        assertThat(result.getTitle()).isEqualTo(movie.getTitle());
        assertThat(result.getDescription()).isEqualTo(movie.getDescription());
        assertThat(result.getDirector()).isEqualTo(movie.getDirector());
        assertThat(result.getGenres()).isEqualTo(movie.getGenres());
        assertThat(result.getActors()).isEqualTo(movie.getActors());

        var imageMapper = new ImageMapperImpl();
        var imageDtos = movie.getImages().stream().map(imageMapper::toDto).toList();

        assertThat(imageDtos).isEqualTo(result.getImages());
    }

    @Test
    public void testDeleteMovie() {
        MovieDo movie = MovieDo.builder()
                .title("Example Movie")
                .description("Action")
                .director("John Doe").
                genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.THRILLER))
                .actors(Arrays.asList("Actor1", "Actor2"))
                .images(Arrays.asList(
                        ImageDo.builder().build(),
                        ImageDo.builder().build()))
                .build();

        when(movieRepository.getReferenceById(movie.getId())).thenReturn(movie);

        movieService.deleteMovie(movie.getId());

        verify(movieRepository, times(1)).getReferenceById(movie.getId());
        verify(movieRepository, times(1)).delete(movie);
    }

    @Test
    public void testUpdateMovie() {
        MovieDo movieDo = MovieDo.builder()
                .title("Example Movie")
                .description("Action")
                .director("John Doe").
                genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.THRILLER))
                .actors(Arrays.asList("Actor1", "Actor2"))
                .build();

        var x = ImageDo.builder().movieId(movieDo.getId()).movie(movieDo).caption("Naked banana").build();
        x.setId(0L);
        var z = ImageDo.builder().movieId(movieDo.getId()).movie(movieDo).caption("Naked apple").build();
        z.setId(1L);

        movieDo.setImages(Arrays.asList(x, z));

        MovieDto movieDto = movieMapper.toDto(movieDo);
        movieDto.setDescription("Drama");

        when(movieRepository.getReferenceById(movieDo.getId())).thenReturn(movieDo);

        movieService.updateMovie(movieDto.getId(), movieDto);

        verify(movieRepository, times(1)).getReferenceById(movieDo.getId());

        assertThat(movieDo.getDescription()).isEqualTo(movieDto.getDescription());
    }
}