package cz.muni.fi.pa165;

import cz.muni.fi.pa165.rating.*;
import lombok.Getter;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

@SpringBootTest(classes = CoreApplication.class)
@AutoConfigureMockMvc
@ExtendWith(SpringExtension.class)
public class RatingServiceTest {

    @Getter
    private final RatingMapper ratingMapper;

    @Mock
    private RatingRepository ratingRepository;

    @InjectMocks
    private RatingService ratingService;

    @Autowired
    public RatingServiceTest(RatingMapper ratingMapper) {
        this.ratingMapper = ratingMapper;
    }

    @BeforeEach
    public void setUp() {
        ratingService = new RatingService(ratingRepository, new RatingMapperImpl());
    }

    @Test
    public void testGetAllRatings() {
        RatingDo ratingFun = new RatingDo(1L, 1L, RatingCategoryEnum.FUN, 6);
        RatingDo ratingCast = new RatingDo(1L, 1L, RatingCategoryEnum.CAST, 2);

        when(ratingRepository.findAll()).thenReturn(List.of(ratingFun, ratingCast));

        List<RatingDto> result = ratingService.getRatings();

        verify(ratingRepository, times(1)).findAll();

        assertThat(result).hasSize(2);
        Optional<RatingDto> ratingFunResultOptional = result.stream().filter(dto -> dto.getCategory() == RatingCategoryEnum.FUN).findFirst();
        assertThat(ratingFunResultOptional.isEmpty()).isFalse();
        Optional<RatingDto> ratingCastResultOptional = result.stream().filter(dto -> dto.getCategory() == RatingCategoryEnum.CAST).findFirst();
        assertThat(ratingCastResultOptional.isEmpty()).isFalse();

        RatingDto ratingFunResult = ratingFunResultOptional.get();
        assertThat(ratingFunResult.getId()).isEqualTo(ratingFun.getId());
        assertThat(ratingFunResult.getRating()).isEqualTo(ratingFun.getRating());
        assertThat(ratingFunResult.getCategory()).isEqualTo(ratingFun.getCategory());
        assertThat(ratingFunResult.getMovieId()).isEqualTo(ratingFun.getMovieId());
        assertThat(ratingFunResult.getUserId()).isEqualTo(ratingFun.getUserId());

        RatingDto ratingCastResult = ratingCastResultOptional.get();
        assertThat(ratingCastResult.getId()).isEqualTo(ratingCast.getId());
        assertThat(ratingCastResult.getRating()).isEqualTo(ratingCast.getRating());
        assertThat(ratingCastResult.getCategory()).isEqualTo(ratingCast.getCategory());
        assertThat(ratingCastResult.getMovieId()).isEqualTo(ratingCast.getMovieId());
        assertThat(ratingCastResult.getUserId()).isEqualTo(ratingCast.getUserId());
    }

    @Test
    public void testCreateRanking() {
        var ratingDto = new RatingDto();
        ratingDto.setMovieId(1L);
        ratingDto.setUserId(1L);
        ratingDto.setRating(6);
        ratingDto.setCategory(RatingCategoryEnum.FUN);

        ratingService.createRating(ratingDto);

        verify(ratingRepository, Mockito.times(1)).save(Mockito.any(RatingDo.class));
    }

    @Test
    public void testGetRatingById() {
        RatingDo rating = new RatingDo(1L, 1L, RatingCategoryEnum.FUN, 6);

        when(ratingRepository.getReferenceById(rating.getId())).thenReturn(rating);

        RatingDto result = ratingService.getById(rating.getId());

        verify(ratingRepository, times(1)).getReferenceById(rating.getId());

        assertThat(result.getId()).isEqualTo(rating.getId());
        assertThat(result.getRating()).isEqualTo(rating.getRating());
        assertThat(result.getCategory()).isEqualTo(rating.getCategory());
        assertThat(result.getMovieId()).isEqualTo(rating.getMovieId());
        assertThat(result.getUserId()).isEqualTo(rating.getUserId());
    }

    @Test
    public void testDeleteRating() {
        RatingDo rating = new RatingDo(1L, 1L, RatingCategoryEnum.FUN, 6);

        when(ratingRepository.getReferenceById(rating.getId())).thenReturn(rating);

        ratingService.deleteRating(rating.getId());

        verify(ratingRepository, times(1)).getReferenceById(rating.getId());
        verify(ratingRepository, times(1)).delete(rating);
    }

    @Test
    public void testUpdateRanking() {
        RatingDo ratingDo = new RatingDo(1L, 1L, RatingCategoryEnum.FUN, 6);
        RatingDto ratingDto = ratingMapper.toDto(ratingDo);
        ratingDto.setRating(2);

        when(ratingRepository.getReferenceById(ratingDo.getId())).thenReturn(ratingDo);

        ratingService.updateRating(ratingDto.getId(), ratingDto);

        verify(ratingRepository, times(1)).getReferenceById(ratingDo.getId());

        assertThat(ratingDo.getRating()).isEqualTo(ratingDto.getRating());
    }
}