package cz.muni.fi.pa165;

import cz.muni.fi.pa165.user.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

@SpringBootTest(classes = CoreApplication.class)
@AutoConfigureMockMvc
@ExtendWith({SpringExtension.class, MockitoExtension.class})
public class UserServiceTest {

    private static final String USERNAME = "Peter";
    private static final String PASSWORD = "password123456";

    @Mock
    private UserRepository userRepository;

    @InjectMocks
    private UserService userService;

    @BeforeEach
    public void setUp() {
        userService = new UserService(userRepository, new UserMapperImpl());
    }

    @Test
    public void testGetUserByUsername() {
        UserDo userDo = new UserDo();
        userDo.setUsername(USERNAME);
        userDo.setPassword(PASSWORD);

        when(userRepository.findByUsername(USERNAME)).thenReturn(userDo);

        UserDto result = userService.findByUsername(USERNAME);

        verify(userRepository, times(1)).findByUsername(USERNAME);
        assertThat(result).isNotNull();
        assertThat(result.getUsername()).isEqualTo(USERNAME);
        assertThat(result.getPassword()).isEqualTo(PASSWORD);
    }

    @Test
    public void testCreateUser() {
        UserDto userDto = new UserDto();
        userDto.setUsername(USERNAME);
        userDto.setPassword(PASSWORD);

        userService.createUser(userDto);

        Mockito.verify(userRepository, Mockito.times(1)).save(Mockito.any(UserDo.class));
    }

    @Test
    public void testUpdateUser() {
        UserDo userDo = new UserDo();
        userDo.setUsername(USERNAME);
        userDo.setPassword(PASSWORD);

        UserDto userDto = new UserDto();
        userDto.setPassword("newPassword");

        when(userRepository.getReferenceById(userDo.getId())).thenReturn(userDo);

        userService.updateUser(userDo.getId(), userDto);

        verify(userRepository, times(1)).getReferenceById(userDo.getId());

        assertThat(userDo.getPassword()).isEqualTo(userDto.getPassword());
    }

    @Test
    public void testDeleteUser() {
        UserDo userDo = new UserDo();
        userDo.setUsername(USERNAME);
        userDo.setPassword(PASSWORD);

        when(userRepository.getReferenceById(userDo.getId())).thenReturn(userDo);

        userService.deleteUser(userDo.getId());

        verify(userRepository, times(1)).getReferenceById(userDo.getId());
        verify(userRepository, times(1)).delete(userDo);
    }
}