package cz.muni.fi.pa165;

import cz.muni.fi.pa165.movie.MovieDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.server.resource.introspection.OAuth2IntrospectionAuthenticatedPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/recommender")
public class RecommenderController {

    private final RecommenderService recommenderService;

    @Autowired
    public RecommenderController(RecommenderService recommenderService) {
        this.recommenderService = recommenderService;
    }

    @Operation(summary = "Get movie by title",
            security = @SecurityRequirement(name = RecommenderApplication.SECURITY_SCHEME_NAME, scopes = {"test_read"}),
            responses = {
                    @ApiResponse(responseCode = "200", description = "OK - movie was successfully returned"),
                    @ApiResponse(responseCode = "401", description = "Unauthorized - access token not provided or valid", content = @Content()),
                    @ApiResponse(responseCode = "403", description = "Forbidden - access token does not have scope test_read", content = @Content()),
            }
    )
    @GetMapping("{id}")
    public List<MovieDto> getSimilarMovies(@PathVariable long id, HttpServletRequest request) {
        String authorizationHeader = request.getHeader("Authorization");
        String accessToken = authorizationHeader.substring(7);
        return recommenderService.getSimilarMovies(id, accessToken);
    }
}
